import { join } from '../fn.js'
import { alpha, alphanumeric, any, char, digit, list, many, map, maybe, noCaseString, separated, seq, skip, string, until } from '../parser.js'
import { Alias, Identifier, Literal, ObjectPath } from './types.js'

export const Symbol = Object.freeze({
  Bracket: Object.freeze({
    Angle: Object.freeze({
      Left: char('<'),
      Right: char('>')
    }),
    Curly: Object.freeze({
      Left: char('{'),
      Right: char('}'),
    }),
    Round: Object.freeze({
      Left: char('('),
      Right: char(')'),
    }),
    Square: Object.freeze({
      Left: char('['),
      Right: char(']'),
    }),
  }),
  Colon: char(':'),
  Comma: char(','),
  Hyphen: char('-'),
  Newline: char('\n'),
  Period: char('.'),
  Plus: char('+'),
  Space: char(' '),
  Quote: char('"')
})

export const word = value => seq(skip(ws), value, skip(ws))
export const collect = parser => map(join(''), parser)
export const quoted = collect(seq(skip(Symbol.Quote), until(Symbol.Quote), skip(Symbol.Quote)))
export const number = seq(digit, many(digit))
export const signed = seq(maybe(any(Symbol.Plus, Symbol.Hyphen)), number)
export const ws = skip(many(any(Symbol.Newline, Symbol.Space)))
export const identifier = map(([x]) => new Identifier(x), collect(seq(alpha, many(alphanumeric))))

const float = map(([n]) => parseFloat(n, 10), collect(seq(signed, Symbol.Period, number)))
const integer = map(([n]) => parseInt(n, 10), collect(signed))
const str = quoted

const toBoolean = v => v === 'false' ? false : true
const boolean = map(([v]) => toBoolean(v), any(string('true'), string('false')))

const property = seq(skip(Symbol.Period), identifier)
const accessor = map(x => new ObjectPath(...x), seq(identifier, property, many(property)))

export const literal = map(([x]) => new Literal(x), any(float, integer, str, boolean))
export const baseValue = any(literal, accessor, identifier)

const as = noCaseString('as')
export const alias = map(([value, name]) => new Alias(value, name), seq(baseValue, word(skip(as)), identifier))
export const value = any(alias, baseValue)

