import { construct, head, ifElse, is, pipe, prepend } from '../fn.js'
import { map, maybe, seq } from '../parser.js'
import { word } from './common.js'
import { matchClause } from './match.js'
import { returnClause } from './return.js'
import { Query, SelectedGraph } from './types.js'
import { useClause } from './use.js'

const hasUseClause = pipe(
  head,
  is(SelectedGraph)
)

const constructQuery = construct(Query)
const noUseQuery = pipe(prepend(undefined), constructQuery)

const collect = ifElse(
  hasUseClause,
  constructQuery,
  noUseQuery
)

export const query = map(
  collect,
  seq(
    maybe(useClause),
    word(matchClause),
    word(returnClause)
  )
)

